/*
 * dvb2010 for linux-dvb
 *
 * Copyright (c) 2008 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE for details.
 */

#include "dvb2010.h"
#include "cstring.h"

/*
 * copy and convert cstring
 */
string
cstring::iconvcpy(char *from, string charset)
{
	iconv_t cd;
	char to[cstring::STRLEN], *instr, *outstr;
	size_t inbytes, outbytes, err;
		
	if (strcmp("UTF-8", charset.c_str()) == 0)
		return from;

	cd = iconv_open("UTF-8", charset.c_str());
	if (cd == (iconv_t)-1)
		return from;

	instr = &from[0];
	inbytes = strlen(from);

	outstr = &to[0];
	outbytes = STRLEN;

	err = iconv(cd, &instr, &inbytes, &outstr, &outbytes);
	iconv_close(cd);
	if (err)
		return from;

	to[cstring::STRLEN - outbytes] = '\0'; /* XXX FIXME bounds */

	return to;
}

/*
 * ETSI EN 300 468 V1.7.1
 *
 * Annex A -> UTF-8
 */
string
cstring::annexa(unsigned char *cbuf, int len)
{
	unsigned char charset = 0;
	stringstream cs;

	int i = 0, j = 0;
	char cstr[cstring::STRLEN];
	string to;

	if (!len || !cbuf[0]) {
		dprintf(25, "EMPTY STRING len = %d, str[0] = %02x\n", len, cbuf[0]);
		return to;
	}
	cs << "ISO8859-1";
	if (cbuf[0] >= 0x01 && cbuf[0] <= 0x0b)
	{	
		charset = cbuf[0];
		cs.str("");
		cs << "ISO8859-" << (charset + 4);
		i = 1;
	}
	else if (cbuf[0] == 0x10)
	{
		if (len < 3 || cbuf[1] != 0x00)
			return to;
		charset = cbuf[2];
		if (charset == 0 || charset > 0x0f)
		{
			/* RESERVED */
			dprintf(25, "CHARSET = RESERVED(%d)\n", charset);
			return to;
		}
		cs.str("");
		cs << "ISO8859-" << charset;
		i = 3;
	}
	else if (cbuf[0] >= 0x11 && cbuf[0] <= 0x15)
	{
		/* WIDE CHAR */
		charset = cbuf[0];
		dprintf(25, "CHARSET = WIDE(%d)\n", charset);
		return to;
	}
	else if (cbuf[0] >= 0x16 && cbuf[0] <= 0x1f)
	{
		/* RESERVED */
		charset = cbuf[0];
		dprintf(25, "CHARSET = RESERVED(%d)\n", charset);
		return to;
	}
	dprintf(25, "CHARSET = %s(%d)\n", cs.str().c_str(), charset);

	if (charset <= 0x0F) /* */
	{
		while (i < len)
		{
			if (cbuf[i] >= 0x80 && cbuf[i] <= 0x9f)
			{
				if (cbuf[i] == 0x8a)
					cstr[j++] = '\n';
				i++;
			}
			else if (cbuf[i] < ' ')
			{
				i++;
			}
			else if (cbuf[i] == ' ' || cbuf[i] == 0xa0)
			{
				if (j && (cstr[j-1] != ' ' && cstr[j-1] != '\n'))
					cstr[j++] = ' ';
				i++;
			}
			else
			{
				cstr[j++] = cbuf[i++];
			}
		}
		cstr[j--] = '\0';
		while (j && (cstr[j] == ' ' || cstr[j] == '\n'))
			cstr[j--] = '\0';
	
		to = iconvcpy(cstr, cs.str());
	}
	return to;
}

/*
 * http://canonical.org/~kragen/strlen-utf8.html
 */
int
cstring::utf8len(string s)
{
	int i = 0, j = 0;

	while (s[i])
	{
		if ((s[i++] & 0xc0) != 0x80)
			j++;
	}
	return j;
}

/*
 * copy string padded or truncated to string
 */
string
cstring::stringcpyp(string from, int len)
{
	string to;
	int tolen = 0, i = 0;

	while (from[i] && tolen < len)
	{
		to += from[i];
		if ((from[i++] & 0xc0) != 0x80)
			tolen++;
	}
	while (tolen++ < len)
		to += ' ';

	return to;
}

/*
 * copy filtered string
 */
string
cstring::stringcpyf(string from, const char filter[])
{
	char to[cstring::STRLEN];

	unsigned int j = 0;
	for (unsigned int i = 0; from[i] && j < cstring::STRLEN; i++)
	{
		bool inset = false;
		for (int k = 0; filter[k]; k++)
		{
			if (from[i] == filter[k])
			{
				inset = true;
				break;
			}
		}
		if (inset)
		{
			if (j && to[j - 1] != ' ')
				to[j++] = ' ';
		}
		else
			to[j++] = from[i];
	}
	to[j] = '\0';

	return to;
}
