/*
 * Copyright (c) 2005-2007 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 */

#include "dvb2010.h"
#include "cstring.h"
#include "util.h"
#include "dotconf.h"

const char
Dotconf::DOTCONFIG[] = ".dvb2010/config";

Dotconf::Dotconf(void)
{
	/* Init default */
	dbtype = "None";
	dbname = "";
	dbhost = "";
	dbuser = "";
	dbpass = "";

	char cbuf[cstring::STRLEN];
	char *homedir;
	stringstream dotfile;

	homedir= getenv("HOME");
	if (homedir != NULL)
	{
		dotfile << homedir << "/" << DOTCONFIG;
	}
	else
	{
		string error = "error: missing $HOME environment variable";
		throw error;
	}
	
	if (access(dotfile.str().c_str(), R_OK) == 0)
	{
		ifstream ifs(dotfile.str().c_str(), ios::in | ios::binary);
	
		if (!ifs.is_open())
		{
			string error = (string)"error: cannot open config file [" + dotfile.str() + "]";
			throw error;
		}

		while (true)
		{
			ifs.getline(cbuf, cstring::STRLEN);
			if (ifs.eof())
				break;

			if (strcasestr(cbuf, "DBTYPE=") == cbuf)
				dbtype = &cbuf[strlen("DBTYPE=")];

			else if (strcasestr(cbuf, "DBNAME=") == cbuf)
				dbname = &cbuf[strlen("DBNAME=")];

			else if (strcasestr(cbuf, "DBHOST=") == cbuf)
				dbhost = &cbuf[strlen("DBHOST=")];
            
			else if (strcasestr(cbuf, "DBUSER=") == cbuf)
				dbuser = &cbuf[strlen("DBUSER=")];
			
			else if (strcasestr(cbuf, "DBPASS=") == cbuf)
				dbpass = &cbuf[strlen("DBPASS=")];
		}
		ifs.close();
	}

	dprintf(30, "DBTYPE = %s\n", dbtype.c_str());
	dprintf(30, "DBNAME = %s\n", dbname.c_str());
	dprintf(30, "DBHOST = %s\n", dbhost.c_str());
	dprintf(30, "DBUSER = %s\n", dbuser.c_str());
	dprintf(30, "DBPASS = %s\n", dbpass.c_str());
}
