/*
 * dvb2010 for linux-dvb
 *
 * Copyright (c) 2008 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE for details.
 */

#include "dvbcompat.h"

/* fec_inner to s2api code rate */
fe_code_rate_t
dvbcompat::fec_inner2coderatet(unsigned int fec_inner)
{
	switch (fec_inner) /* TABLE 35 */
	{
		case 1: return FEC_1_2;
		case 2: return FEC_2_3;
		case 3: return FEC_3_4;
		case 4: return FEC_5_6;
		case 5: return FEC_7_8;
		case 6: return FEC_8_9;
		case 7: return FEC_3_5;
		case 8: return FEC_4_5;
		case 9: return FEC_9_10;
		case 15:return FEC_NONE;
		case 0: /* NOT DEFINED */
		case 10:/* RESERVED */
		case 11:/* RESERVED */
		case 12:/* RESERVED */
		case 13:/* RESERVED */
		case 14:/* RESERVED */
		default:break;
	}
	return FEC_AUTO;
}

/* s2api code rate to cstring */
const char *
dvbcompat::coderatet2cstr(fe_code_rate_t coderate)
{
	switch (coderate)
	{
		case FEC_NONE:return "0/0";
		case FEC_1_2: return "1/2";
		case FEC_2_3: return "2/3";
		case FEC_3_4: return "3/4";
		case FEC_3_5: return "3/5";
		case FEC_4_5: return "4/5";
		case FEC_5_6: return "5/6";
		case FEC_6_7: return "6/7";
		case FEC_7_8: return "7/8";
		case FEC_8_9: return "8/9";
		case FEC_9_10:return "9/10";
		default:break;
	}
	return "AUTO";
}

/* string to s2api code rate */
fe_code_rate_t
dvbcompat::str2coderatet(string s)
{
	if (s == "0/0")
		return FEC_NONE;
	else if (s == "1/2")
		return FEC_1_2;
	else if (s == "2/3")
		return FEC_2_3;
	else if (s == "3/4")
		return FEC_3_4;
	else if (s == "5/6")
		return FEC_5_6;
	else if (s == "7/8")
		return FEC_7_8;
	else if (s == "8/9")
		return FEC_8_9;
	else if (s == "3/5")
		return FEC_3_5;
	else if (s == "4/5")
		return FEC_4_5;
	else if (s == "9/10")
		return FEC_9_10;
	else if (s == "6/7")
		return FEC_6_7;
	return FEC_AUTO; /* XXX */
}

/* s2api code rate to szap2/vdr cstring */
const char *
dvbcompat::coderatet2szap(fe_code_rate_t coderate)
{
	switch (coderate)
	{
		case FEC_NONE:return "0";
		case FEC_1_2: return "12";
		case FEC_2_3: return "23";
		case FEC_3_4: return "34";
		case FEC_3_5: return "35";
		case FEC_4_5: return "45";
		case FEC_5_6: return "56";
		case FEC_6_7: return "67";
		case FEC_7_8: return "78";
		case FEC_8_9: return "89";
		case FEC_9_10:return "910";
		default:break;
	}
	return "999";
}

/* szap2/vdr to s2api code rate */
fe_code_rate_t
dvbcompat::szap2coderatet(string s)
{
	if (s == "0")
		return FEC_NONE;
	else if (s == "12")
		return FEC_1_2;
	else if (s == "23")
		return FEC_2_3;
	else if (s == "34")
		return FEC_3_4;
	else if (s == "35")
		return FEC_3_5;
	else if (s == "45")
		return FEC_4_5;
	else if (s == "56")
		return FEC_5_6;
	else if (s == "67")
		return FEC_6_7;
	else if (s == "78")
		return FEC_7_8;
	else if (s == "89")
		return FEC_8_9;
	else if (s == "910")
		return FEC_9_10;
	return FEC_AUTO; /* XXX */
}

/* system to s2api delivery system */
fe_delivery_system_t
dvbcompat::system2delsyst(unsigned int system)
{
	if (system == 1)
		return SYS_DVBS2;

	return SYS_DVBS;
}

/* s2api delivery system to cstring */
const char *
dvbcompat::delsyst2cstr(fe_delivery_system_t delsys)
{
	if (delsys == SYS_DVBS2)
		return "DVB-S2";

	return "DVB-S";
}

/* string to delivery system */
fe_delivery_system_t
dvbcompat::str2delsyst(string s)
{
	if (s == "DVB-S2")
		return SYS_DVBS2;

	return SYS_DVBS;
}

/* s2api delivery system to szap2/vdr cstring */
const char *
dvbcompat::delsyst2szap(fe_delivery_system_t delsys)
{
	if (delsys == SYS_DVBS2)
		return "1";

	return "0";
}

/* szap2/vdr to delivery system */
fe_delivery_system_t
dvbcompat::szap2delsyst(string s)
{
	if (s == "1")
		return SYS_DVBS2;

	return SYS_DVBS;
}

/* polarisation to s2api voltage */
fe_sec_voltage_t
dvbcompat::polarisation2voltaget(char polarisation)
{
	switch (polarisation)
	{
		case 0:
		case 'h':
		case 'H': return SEC_VOLTAGE_18;

		case 1:
		case 'v':
		case 'V': return SEC_VOLTAGE_13;

		case 2:
		case 'l':
		case 'L': return SEC_VOLTAGE_18;

		case 3:
		case 'r':
		case 'R': return SEC_VOLTAGE_13;
		default:break;
	}
	return SEC_VOLTAGE_OFF;
}

/* s2api voltage to cstring */
const char *
dvbcompat::voltaget2cstr(fe_sec_voltage_t voltage)
{
	switch (voltage)
	{
		case SEC_VOLTAGE_18: return "h";
		case SEC_VOLTAGE_13: return "v";
		default:break;
	}
	return "OFF";
}

/* polarisation to uint */
unsigned int
dvbcompat::polarisation2uint(char polarisation)
{
	switch (polarisation)
	{
		case 0:
		case 'h':
		case 'H': return 0;

		case 1:
		case 'v':
		case 'V': return 1;

		case 2:
		case 'l':
		case 'L': return 2;

		case 3:
		case 'r':
		case 'R': return 3;
		default:break;
	}
	return 0; /* XXX */
}

/* polarisation to cstring */
const char *
dvbcompat::polarisation2cstr(unsigned int polarisation, bool upperCase)
{
	switch (polarisation) {
	case 0: return ((upperCase) ? ("H") : ("h"));
	case 1: return ((upperCase) ? ("V") : ("v"));
	case 2: return ((upperCase) ? ("L") : ("l"));
	case 3: return ((upperCase) ? ("R") : ("r"));
	default:break;
	}

	return "?";
}

/* modulation to s2api modulation */
fe_modulation_t
dvbcompat::modulation2modulationt(unsigned int modulation)
{
	if (modulation == 2)
		return PSK_8;
	
	return QPSK;
}

/* s2api modulation to cstring */
const char *
dvbcompat::modulationt2cstr(fe_modulation_t modulation)
{
	if (modulation == PSK_8)
		return "8PSK";

	return "QPSK";
}

/* string to modulationt */
fe_modulation_t
dvbcompat::str2modulationt(string s)
{
	if (s == "8PSK")
		return PSK_8;
	
	return QPSK;
}

/* s2api modulation to szap2/vdr cstring */
const char *
dvbcompat::modulationt2szap(fe_modulation_t modulation)
{
	if (modulation == PSK_8)
		return "5";
	
	return "2";
}
 
/* szap2/vdr to modulationt */
fe_modulation_t
dvbcompat::szap2modulationt(string s)
{
	if (s == "5")
		return PSK_8;
	
	return QPSK;
}

/* roll-off to s2api roll-off */
fe_rolloff_t
dvbcompat::rolloff2rollofft(unsigned int rolloff)
{
	switch (rolloff)
	{
		case 0: return ROLLOFF_35;
		case 1: return ROLLOFF_25;
		case 2: return ROLLOFF_20;
		default:break;
	}
	return ROLLOFF_35;
}

/* s2api roll-off to cstring */
const char *
dvbcompat::rollofft2cstr(fe_rolloff_t rolloff)
{
	switch (rolloff)
	{
		case ROLLOFF_35: return "0.35";
		case ROLLOFF_25: return "0.25";
		case ROLLOFF_20: return "0.20";
		default:break;
	}
	return "0.00";
}

/* string to rollofft */
fe_rolloff_t 
dvbcompat::str2rollofft(string s)
{
	if (s == "0.25")
		return ROLLOFF_25;
	else if (s == "0.20")
		return ROLLOFF_20;
		
	return ROLLOFF_35;
}

/* s2api roll-off to szap2/vdr cstring */
const char *
dvbcompat::rollofft2szap(fe_rolloff_t rolloff)
{
	switch (rolloff)
	{
		case ROLLOFF_35: return "35";
		case ROLLOFF_25: return "25";
		case ROLLOFF_20: return "20";
		default:break;
	}
	return "0";
}

/* szap2/vdr to rollofft */
fe_rolloff_t 
dvbcompat::szap2rollofft(string s)
{
	if (s == "25")
		return ROLLOFF_25;
	else if (s == "20")
		return ROLLOFF_20;
		
	return ROLLOFF_35;
}

/* pilot to s2api pilot */
fe_pilot_t
dvbcompat::pilot2pilott(unsigned int pilot)
{
	switch (pilot)
	{
		case 0: return PILOT_OFF;
		case 1: return PILOT_ON;
		default:break;
	}
	return PILOT_AUTO;
}

/* s2api pilot to cstring */
const char *
dvbcompat::pilott2cstr(fe_pilot_t pilot)
{
	switch (pilot)
	{
		case PILOT_OFF: return "OFF";
		case PILOT_ON:  return "ON";
		default:break;
	}
	return "AUTO";
}

/* string to pilot2 */
fe_pilot_t 
dvbcompat::str2pilott(string s)
{
	if (s == "OFF")
		return PILOT_OFF;
	else if (s == "ON")
		return PILOT_ON;
		
	return PILOT_AUTO;
}
