/*
 * Copyright (c) 2008 Darron M Broad
 * All rights reserved.
 */

#ifndef _MATRIX_H
#define _MATRIX_H

/*
 * 2d matrix array
 *
 * If size is known on creation use constructor (x,y) else
 * use default constructor then call 'resize' later.
 */
template <class T>
class Matrix
{
private:
	unsigned int dimX, dimY, nxtX, nxtY;
	T *data;

public:
	Matrix(void)
	{
		dimX = dimY = nxtX = nxtY = 0;
		data = NULL;
	}
	
	Matrix(unsigned int x, unsigned int y)
	{
		if (x == 0 || y == 0)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << x << "," << y << ")" << endl;
			throw;
		}
		nxtX = nxtY = 0;
		data = new T[(dimX = x) * (dimY = y)];
	}
	
	~Matrix()
	{
		if (dimX)
			delete[] data;
	}

	Matrix(const Matrix &matrix)
	{
		dimX = matrix.dimX;
		dimY = matrix.dimY;
		nxtX = matrix.nxtX;
		nxtY = matrix.nxtY;
		data = NULL;
		if (dimX)
		{
			unsigned int size = dimX * dimY;

			data= new T[size];

			for (unsigned int i = 0; i < size; i++)
				data[i] = matrix.data[i];
		}
	}

	void resize(unsigned int x, unsigned int y)
	{
		if (dimX)
			delete[] data;

		if (x == 0 || y == 0)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << x << "," << y << ")" << endl;
			throw;
		}
		nxtX = nxtY = 0;
		data = new T[(dimX = x) * (dimY=y)];
	}

	unsigned int getdimx(void)
	{
		return dimX;
	}
	
	unsigned int getdimy(void)
	{
		return dimY;
	}

	/*
	 * absolute
	 */
	void set(unsigned int x, unsigned int y, T val)
	{
		if (x >= dimX || y >= dimY)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << x << "," << y << ") [" << dimX << "," << dimY << "]" << endl;
			throw;
		}
		data[x * dimY + y] = val;
	}

	T get(unsigned int x, unsigned int y)
	{
		if (x >= dimX || y >= dimY)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << x << "," << y << ") [" << dimX << "," << dimY << "]" << endl;
			throw;
		}
		return data[x * dimY + y];
	}
	
	/*
	 * cursor
	 */
	void setxy(unsigned int x, unsigned int y)
	{
		if (x >= dimX || y >= dimY)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << x << "," << y << ") [" << dimX << "," << dimY << "]" << endl;
			throw;
		}
		nxtX = x;
		nxtY = y;
	}
	
	void setx(unsigned int x)
	{
		if (x >= dimX)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << x << ") [" << dimX << "]" << endl;
			throw;
		}
		nxtX = x;
	}
	
	void sety(unsigned int y)
	{
		if (y >= dimY)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << y << ") [" << dimY << "]" << endl;
			throw;
		}
		nxtY = y;
	}
	
	T getxy(void)
	{
		return data[nxtX * dimY + nxtY];
	}
	
	T getx(unsigned int x)
	{
		if (x >= dimX)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << x << ") [" << dimX << "]" << endl;
			throw;
		}
		return data[x * dimY + nxtY];
	}
	
	T gety(unsigned int y)
	{
		if (y >= dimY)
		{
			cerr << __PRETTY_FUNCTION__ << " (" << y << ") [" << dimY << "]" << endl;
			throw;
		}
		return data[nxtX * dimY + y];
	}
};

#endif /* ! _MATRIX_H */
