/*
 * Copyright (c) 2004-2008 Darron M Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE for details.
 */

#ifndef _SQL_H
#define _SQL_H

class SqlMatrix : public Matrix<string>
{
private:
	vector<string> fieldnames;
	unsigned int getColumn(string fieldname)
	{
		return distance(fieldnames.begin(), find(fieldnames.begin(), fieldnames.end(), fieldname));
	}

protected:

public:
	void addFieldName(string fieldname)
	{
		fieldnames.push_back(fieldname);
	}
	string getFieldName(int col)
	{
		return fieldnames[col];
	}
	string asString(string fieldname, unsigned int row)
	{
		return get(getColumn(fieldname), row);
	}
	int asInt(string fieldname, unsigned int row)
	{
		return util::toInt(get(getColumn(fieldname), row));
	}
	double asDouble(string fieldname, unsigned int row)
	{
		return util::toDouble(get(getColumn(fieldname), row));
	}
	char asChar(string fieldname, unsigned int row)
	{
		return get(getColumn(fieldname), row)[0];
	}
};

class Sql
{
private:
	static Sql *handle;

protected:
	Sql(void);
	~Sql(void);

public:
	static Sql *getHandle(bool = false);

	virtual void reconnect(void) = 0;

	virtual void SQLcommand(const char *) = 0;
	virtual void SQLcommand(string) = 0;
	virtual void SQLcommand(stringstream &) = 0;

	virtual int SQLcount(const char *) = 0;
	virtual int SQLcount(string) = 0;
	virtual int SQLcount(stringstream &) = 0;

	virtual int SQLquery(const char *, SqlMatrix *) = 0;
	virtual int SQLquery(string, SqlMatrix *) = 0;
	virtual int SQLquery(stringstream &, SqlMatrix *) = 0;

	virtual string SQLescape(const char *) = 0;
	virtual string SQLescape(string) = 0;

	virtual string SQLnull(string) = 0;
	virtual string SQLquotefield(string) = 0;

	void file(const char *);

	int getId(const char *);
	int getId(string);
	int getId(stringstream &);

	int getSatelliteId(class Satellite *);
	int addSatellite(class Satellite *);
	int updateSatellite(class Satellite *);

	int getTransponderId(class Transponder *);
	int addTransponder(int, class Transponder *);
	int updateTransponder(class Transponder *);
	int updateTransponderDefaultUd(class Transponder *);
	int updateTransponderDesc(class Transponder *);

	int addTransponderAndServices(class Satellite *, class Transponder *);
	
	int getServiceId(class Service *);
	int addService(int, class Service *);
	int updateService(class Service *);
	int updateServiceDefaultUd(class Service *);

	int getSatellites(SqlMatrix *);
	int getTransponders(int, SqlMatrix *);
	int getServices(int, SqlMatrix *);

	/* new instance from db */
	class Service newService(SqlMatrix *, unsigned int);
	class Transponder newTransponder(SqlMatrix *, unsigned int);
	class Satellite newSatellite(SqlMatrix *, unsigned int);

	/* unserialise from database */
	class Satellite getSatellite(SqlMatrix *, unsigned int);
};

#endif /* !_SQL_H */
