/*
 * dvb2010 for linux-dvb
 *
 * Copyright (c) 2008 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE for details.
 */
#ifndef _TRANSPONDER_H
#define _TRANSPONDER_H

#define SECTION_LENGTH(X)	(((X[1]  &0x0f) << 8) + X[2] +3)
#define INFO_LENGTH(X,n)	(((X[(n)]&0x0f) << 8) + X[(n)+1])
#define ID_13(X,n)		(((X[(n)]&0x1f) << 8) + X[(n)+1])
#define ID_16(X,n)		(((X[(n)])      << 8) + X[(n)+1])

#define HEXADJ ('A' - '9' - 1)

class Pmt
{
private:

protected:

public:
	static const unsigned int MAX = 8192;

	Pmt(__u16 pid)
	{
		this->pid = pid;
		refcount = 1;
	}
	__u16 pid;
	unsigned int refcount;
};

class Transponder
{
private:
	static const unsigned int CRC_LENGTH = 4;

	vector<class Pmt> pmt;

	int read(class Satellite *, unsigned char *, unsigned char, unsigned int, unsigned int *);

	void addtag(unsigned char, unsigned char *, int, int, int);
	void readdesc(unsigned char, unsigned char *, int);

	unsigned int findservice(int);
	unsigned int addservice(int, int = 0);
	
	unsigned int findpmt(int);
	unsigned int addpmt(int);

protected:

public:
	/* Table Ids */
	static const unsigned char PAT  = 0x00;
	static const unsigned char PMT  = 0x02;
	static const unsigned char ANIT = 0x40;
	static const unsigned char ONIT = 0x41;
	static const unsigned char SDT  = 0x42;
	
	int readsdt(class Satellite *);
	int readpmt(class Satellite *);
	int readpat(class Satellite *);
	int readnit(class Satellite *, unsigned char);

	Transponder(void)
	{
		frequency = 0;
		symbol_rate = 0;
		rolloff = ROLLOFF_35;
		voltage = SEC_VOLTAGE_OFF;
		coderate = FEC_AUTO;
		delsys = SYS_DVBS;
		modulation = QPSK;
		pilot = PILOT_AUTO;
		polarisation = 0;
		transponderid = -1;
		satelliteid = -1;
		nit = 0x10;
		nid = 0;
		onid = 0;
		tid = 0;
	}
	friend bool operator < (Transponder &left, Transponder &right)
	{
		if (left.frequency < right.frequency)
			return true;
		if (left.frequency > right.frequency)
			return false;
		if (left.voltage < right.voltage)
			return true;
		return false;
	}
	__u32 frequency;
	__u32 symbol_rate;
	fe_rolloff_t rolloff;
	fe_sec_voltage_t voltage;
	fe_code_rate_t coderate;
	fe_delivery_system_t delsys;
	fe_modulation_t modulation;
	fe_pilot_t pilot;
	unsigned int polarisation;

	/* User-defined name */
	string name;

	int transponderid, satelliteid;

	string network;
	unsigned int nit, nid, onid, tid;
	
	vector<class Service> service;
};

#endif /* ! _TRANSPONDER_H */
