/*
 * Copyright (c) 2005-2008 Darron M Broad
 * All rights reserved.
 */

#include "dvb2010.h"
#include "cstring.h"
#include "util.h"

/*
 * Stuff that probably ought to be somewhere else
 */

/******************************************************************************
 * INT / FLOAT
 *****************************************************************************/

/*
 * to integer (base 10)
 */
int
util::toInt(const char *s)
{
	return strtol(s, NULL, 10);
}
int
util::toInt(string s)
{
	return strtol(s.c_str(), NULL, 10);
}

/*
 * to integer (any base)
 */
int
util::toInt(const char *s, int base)
{
	return strtol(s, NULL, base);
}
int
util::toInt(string s, int base)
{
	return strtol(s.c_str(), NULL, base);
}

/*
 * to double
 */
double
util::toDouble(const char *s)
{
	return strtod(s, NULL);
}
double
util::toDouble(string s)
{
	return strtod(s.c_str(), NULL);
}

/******************************************************************************
 * STRING
 *****************************************************************************/

/*
 * to string (any base)
 */
string
util::toString(int n, int base)
{
	string str = "", sign = "";
	char mod;

	if (n == 0)
		return "0";
	if (n < 0)
	{
		n = abs(n);
		sign = "-";
	}
	while (n)
	{
		mod = '0' + (n % base);
		if (mod > '9')
			mod += ('A' - '9' - 1);
		str = mod + str;
		n /= base;
	}
	return sign + str;
}
string
util::toString(int n)
{
	return util::toString(n, 10);
}

/*
 * timestamp
 */
string
util::timestamp(void)
{
	time_t now;
	time(&now);
	struct tm *tim = localtime(&now);
	stringstream ss;
	
	ss << setfill('0');
	ss << setw(4) << 1900 + tim->tm_year;
	ss << setw(2) << 1 + tim->tm_mon;
	ss << setw(2) << tim->tm_mday;
	ss << setw(2) << tim->tm_hour;
	ss << setw(2) << tim->tm_min;
	ss << setw(2) << tim->tm_sec;

	return ss.str();
}

/*
 * split string to vector
 */
vector<string>
util::split(string line, char splitchar, unsigned int first)
{
	vector<string> split;
	string substring;
	int last;
	unsigned int length= line.length();

	while (first < length)
	{
		last = line.find(splitchar, first);
		if (last < 0)
			last = length;
		substring = line.substr(first, last - first);
		split.push_back(substring);
		first = last + 1;
	}
	return split;
}

/******************************************************************************
 * REGEX
 *****************************************************************************/

/*
 * prepare regex
 */
void
util::regexPrepare(regex_t *re, string pat)
{
	if (regcomp(re, pat.c_str(), REG_EXTENDED | REG_ICASE ) != 0)
		throw "Can't compile expression";
}

/*
 * do regex
 */
vector<string>
util::regex(regex_t *re, string &str)
{
	vector<string> matches;
	regmatch_t rm;
	int cflags = 0;
	string nomatch;

	while ((!regexec(re, str.c_str(), 1, &rm, cflags)) && (rm.rm_eo > rm.rm_so))
	{
		string match = str.substr(rm.rm_so, rm.rm_eo - rm.rm_so);
		nomatch += str.substr(0, rm.rm_so);
		matches.push_back(match);
		str = str.substr(rm.rm_so + match.length());
		cflags = REG_NOTBOL;
	}
	str = nomatch + str;

	return matches;
}

/******************************************************************************
 * SATELLITE
 *****************************************************************************/

/*
 * Satellite angular position
 *	Eg. convert 28.2E => 282
 */
int
util::cstringToAngle(char *cstr)
{
	int len = strlen(cstr) - 1;
	if (len <= 0 || len >= (int)cstring::STRMAX)
		return INT_MAX;

	char c = toupper(cstr[len]);
	if (c != 'E' && c != 'W')
		return INT_MAX;

	int angle = (int) (10.0 * strtod(cstr, NULL) + 0.5);

	if (c == 'W')
		angle = 0 - angle;

	if (angle < -1799 || angle > 1800)
		return INT_MAX;

	return angle;
}

/*
 * reverse of above
 */
string
util::angleToString(int angle)
{
	setlocale(LC_ALL, "POSIX"); 

	stringstream ss;
	ss << setiosflags(ios::fixed) << setprecision(1);

	if (angle >= 0)
		ss << ((float)angle / 10.0) << "E";
	else
		ss << (0.0 - (float)angle / 10.0) << "W";

	setlocale(LC_ALL, "");

	return ss.str();
}
