/*
 * dvb2010 for linux-dvb
 *
 * Copyright (c) 2008 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE for details.
 */

#include "dvb2010.h"
#include "cstring.h"
#include "util.h"
#include "lnb.h"
#include "satellite.h"
#include "transponder.h"
#include "service.h"
#include "zaptune.h"

void
usage(char *execname, string errstr, int errnum)
{
	cerr << "USAGE: " << execname << " [OPTIONS] [ZAPFILE] 'IDENT'\n\n";
	cerr << "Interface options:\n"
		" -a num\t\tDVB-S/S2 adapter num.\n"
		" -d n,b\t\tDiSEqC repeat count n, burst disable (0) or\n\t\tburst enable (1).\n"
		" -l l,h,s\tLNBF lo-band LO l, hi-band LO h & switch-over\n\t\tfreq s in MHz.\n"
		" -w sec\t\tWait up to sec seconds for tuning lock.\n\n"
		" (default)\t-a 0 -d 2,1 -l 9750,10600,11700 -w 5\n\n";

	cerr << "Output option:\n"
		" -r cmd\t\tExec 'cmd /dev/dvb/adapter[num]/dvr0'\n\n";

	cerr << "FILES:\n"
		" " << getenv("HOME") << "/.dvb2010/channels.conf\n\t\tDefault szap-s2 compatible channels.conf else use ZAPFILE.\n\n";

	cerr << "EXAMPLES:\n"
		" " << execname << " -r \"mplayer -vf pp=lb -fs\" channels.conf \"BBC 1\"\n\t\tTune to BBC1 and watch in mplayer.\n\n";
	cerr << " " << execname << " -r \"mencoder -ovc copy -oac copy -o bbc1.avi -endpos 10\" \"BBC 1\"\n\t\t"
		"Tune to BBC1 and record with mencoder for 10 seconds to bbc1.avi.\n";

	if (errstr != "")
		cerr << "\nERROR:\n " << errstr << ".\n";

	exit(errnum);
}

int
main(int argc, char **argv)
{
	string ident, zapfile, command;
	char c, *execdup, *execname, *comma;
	class Satellite sat;

	execdup = strdup(argv[0]);
	execname = basename(execdup);

	opterr = 0;
	while ((c = getopt(argc, argv, "a:l:w:d:r:")) != -1)
	{
		switch (c)
		{
			case 'a':
				sat.lnb.phy_adapter = util::toInt(optarg);
				break;
			case 'l':
				sat.lnb.low = 1000 * util::toInt(optarg);
				sat.lnb.high = sat.lnb.threshold = 0;
				comma = strstr(optarg, ",");
				if (comma != NULL)
				{
					sat.lnb.high = 1000 * util::toInt(++comma);
					comma = strstr(comma, ",");
					if (comma != NULL)
						sat.lnb.threshold = 1000 * util::toInt(++comma);
				}
				break;
			case 'w':
				sat.lnb.tuning_delay = util::toInt(optarg);
				break;
			case 'd':
				sat.lnb.dsec_repeat = util::toInt(optarg);
				comma = strstr(optarg, ",");
				if (comma != NULL)
					sat.lnb.dsec_burst = util::toInt(++comma);
				break;
			case 'r':
				command = optarg;
				break;
			case '?':
				usage(execname, "", EX_OK);
				break;
			default:
				string err = "Unknown option " + (string)optarg;
				usage(execname, err, EX_DATAERR);
				break;
		}
	}
	argc -= optind;
	argv += optind;
	if (argc < 1)
		usage(execname, "Missing args(s)", EX_DATAERR);
	if (argc == 1)
	{
		zapfile = (string)getenv("HOME") + "/.dvb2010/channels.conf";
		ident   = argv[0];
	}
	else if (argc == 2)
	{
		zapfile = argv[0];
		ident   = argv[1];
	}
	else
                usage(execname, "Too many args(s)", EX_DATAERR);


	setlocale(LC_ALL, "");
	try
	{
		sat.lnb.open(0);
		sat.lnb.tune(&sat, zapfile, ident, command);
	}
	catch (const char *e1)
	{
		cerr << "\nError: " << e1 << endl;
	}
	catch (string e2)
	{
		cerr << "\nError: " << e2 << endl;
	}
	sat.lnb.close();

	exit(EX_OK);
}
